use std::{
    env, fs,
    io::{stdout, Read, Write},
};

use docx_rs::read_docx;
use serde_json::Value;

fn main() -> anyhow::Result<()> {
    let args: Vec<String> = env::args().collect();
    let file_name = &args[1];

    let mut lines: Vec<String> = vec![];

    parse_docx(file_name, &mut lines)?;

    let ascii_upper_limit = u32::from_str_radix("7F", 16).unwrap();
    let mut excluded = vec![];
    for line in fs::read_to_string("excluded")?.lines() {
        if let Some(c) = line.chars().next() {
            let c = format!("{:x}", c as u32);
            let u = u32::from_str_radix(&c, 16)?;
            excluded.push(u);
        }
    }

    let mut lock = stdout().lock();

    for line in lines {
        for c in line.chars() {
            let uc = c as u32;
            if uc > ascii_upper_limit && !excluded.contains(&uc) {
                writeln!(lock, "{c} - {line}")?;
            }
        }
    }

    lock.flush()?;

    Ok(())
}

fn read_to_vec(file_name: &str) -> anyhow::Result<Vec<u8>> {
    let mut buf = Vec::new();
    std::fs::File::open(file_name)?.read_to_end(&mut buf)?;
    Ok(buf)
}

fn parse_docx(file_name: &str, lines: &mut Vec<String>) -> anyhow::Result<()> {
    let data: Value = serde_json::from_str(&read_docx(&read_to_vec(file_name)?)?.json())?;
    if let Some(children) = data["document"]["children"].as_array() {
        children.iter().for_each(|c| read_children(c, lines));
    }
    Ok(())
}

fn read_children(node: &Value, lines: &mut Vec<String>) {
    if let Some(children) = node["data"]["children"].as_array() {
        children.iter().for_each(|child| {
            if child["type"] != "text" {
                read_children(child, lines);
            } else {
                lines.push(child["data"]["text"].to_string());
            }
        });
    }
}
